;;; .emacs.el
;;; By Greg J. Badros, (C) 1994-1996
;;; gjb@cs.duke.edu, http://www.cs.duke.edu/~gjb
;;; $Id: .emacs.el,v 1.123 1996/02/17 19:15:19 gjb Exp $
;;; 
;;; This file has evolved over the past several years.  It includes
;;; parts written by me, written by others, and found from all over
;;; the place.  Use at your own risk.  Distribute freely as long as
;;; credit is given.  Many ideas were taken from Anders Lindgren's
;;; startup file, andersl@csd.uu.se.
;;;

;;{{{ General

;;{{{ Load the default site-file

;;;---------------------------------------------------------------------
;;; Load the default file BEFORE executing this startupfile, so that we
;;; will get the last laught! Also make sure that it isn't read again!
;;;

(load "default" 'noerror 'nomessage)
(setq inhibit-default-init t)

;;}}}

;;; Increase the garbage collection threshold to reduce number of
;;; garbage collections (each could potentially become longer)
(setq gc-cons-threshold 1000000)

;;; Don't show the startup messages
(setq inhibit-startup-echo-area-message "gjb")
(setq inhibit-startup-message 't)

;;; Ask if final newfile is needed
(setq require-final-newline 'ask)

;;; FIX:  What does this do?
;;;(setq print-escape-newlines t)


;;; Fix the load path, but only if we've not fixed it yet
(if (member (expand-file-name "~/emacs") load-path)
    ()
  (setq load-path (append (mapcar 'expand-file-name
				  (list "~/emacs"
					"~/emacs/edb"))
			  load-path)))

;;; Stay at end of line when moving cursor vertically
(setq track-eol t)

;;; Don't create a bunch of garbage files--- version control
;;; accomplished much more, much more nicely
(setq make-backup-files nil)

;;; Don't add new lines when cursor downing at end of buffer
(setq next-line-add-newlines nil)

;;; use text-mode by default
(setq default-major-mode 'text-mode)

;;; delete auto-save files when buffer is explicitly saved
(setq delete-auto-save-files t)

;;; don't ring the annoying bell---visual bell takes a
;;; while to get used to, but is nicer
(setq visible-bell 't)

(setq scroll-step 5)

;;; Let us use M-: for eval-expression command, w/o asking
;;; first to enable the command
(put 'eval-expression 'disabled nil)

;;; put line number in mode line
(line-number-mode t)

(setq display-time-day-and-date t
      display-time-24hr-format t)
;;;(display-time)



;;; better control about marking-- make it visible
(setq transient-mark-mode t)
(setq mark-even-if-inactive t)

;;; highlight incremental searches
(setq-default search-highlight t)

(setq default-frame-alist (cons '(minibuffer . t) default-frame-alist))

;;;(setq default-frame-alist (cons '(height . 50) default-frame-alist))

(cond (window-system
       (setq hilit-mode-enable-list  '(not text-mode)
             hilit-background-mode   'light
             hilit-inhibit-hooks     nil
             hilit-inhibit-rebinding nil)
;;       (setq hilit-mode-enable-list 
;;	     '(not emacs-lisp-mode lisp-mode latex-mode tex-mode))
       (require 'hilit19)
       (require 'cal-x)
       (require 'mouse-sel)
       (setq mouse-yank-at-point t)
       (global-set-key [mode-line C-mouse-2] 'mouse-tear-off-window)
       (require 'mldrag)
       (global-set-key "\C-z" 'call-last-kbd-macro)
       (global-set-key [mode-line down-mouse-1] 'mldrag-drag-mode-line)
       (global-set-key [vertical-line down-mouse-1] 'mldrag-drag-vertical-line)
       (global-set-key [vertical-scroll-bar S-down-mouse-1]
		       'mldrag-drag-vertical-line)))


;;;------------------------------------------------------------
;;; AucTeX
;;;

(require 'tex-site)

(setq LaTeX-version "2e")

(defun my-TeX-mode-hook ()
  (if window-system
      (font-lock-mode t))
  ;; Make \item's look nice.
  (setq LaTeX-item-indent -6)
  
  ;; Make AUXTeX query for Master File if the current file has no
  ;; \documentclass... 
  (setq TeX-master nil)

  ;; Enable AUCTeX parsing.
  (setq TeX-parse-self t)
  (setq TeX-auto-save t)

  ;; Automatically replace tabs with appropriate count of spaces
  (setq TeX-auto-untabify t)

  ;; This needed if you have directories ending with "//" in your
  ;; TEXINPUTS.
  (setq TeX-macro-private nil)

  ;; I prefer to have my figures as close as possible
  (setq TeX-float "htb")

  ;; I want to keep my old M-g
  (local-set-key "\M-g" 'goto-line)
  (local-set-key [A-tab] 'TeX-complete-symbol))


(add-hook 'TeX-mode-hook 'my-TeX-mode-hook)
(add-hook 'tex-mode-hook  'turn-on-auto-fill)
(setq TeX-default-mode 'latex-mode)


;;;------------------------------------------------------------
;;; Latex help
;;;

(autoload 'latex-help "ltx-help" nil t nil)
(define-key help-map "\C-l" 'latex-help)
(setq latex-help-info-file '(latex2e))

;;}}}

(require 'vc)
(require 'scroll-in-place)
(require 'jka-compr)
(require 'ange-ftp)
(setq ange-ftp-default-user "anonymous")
(setq ange-ftp-generate-anonymous-password t)
;;;(require 'complete)
;;;(require 'icomplete)
(load "rcs-complete")

(if (string= system-type "linux")
    (setq PC-include-file-path
	  '("/usr/include"
	    "/usr/X11/include" 
	    "/usr/include/g++-include"
	    "/usr/src/linux/include/linux"))
  (setq PC-include-file-path
      '("/usr/include"
	"/opt/SUNWspro/SC3.0.1/include/CC" 
	"/opt/SUNWspro/SC3.0.1/include/cc"
	"/usr/openwin/include" 
	"/usr/dt/include/")))

(setq inferior-lisp-program
      (if (string= system-type "linux")
	  "/usr/bin/gcl"
	"/misc/gcl-2.1/bin/gcl"))

(setq diary-file (expand-file-name "~/text/diary"))

(defun start-server ()
  "Start the gnu server process for the current session."
  (interactive)
  (server-start)
;;;  (message "Server started...")
;;;  (sit-for 1)
  ;; Create a *server* buffer for server log messages
  (get-buffer-create "*server*"))

(defun server-new-screen ()
  (select-frame (x-create-frame-with-faces nil)))

(add-hook 'server-visit-hook 'server-new-screen)

(add-hook 'comint-mode-hook 
	  (function
	   (lambda ()
	     (setenv "PAGER" "/bin/cat")
	     (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m)
	     (setq comint-scroll-show-maximum-output 't))))

(setq explicit-shell-file-name "zsh")

;;;(start-server)
(setq shell-file-name "/bin/sh")

(fmakunbound 'c-mode)
(makunbound 'c-mode-map)
(fmakunbound 'c++-mode)
(makunbound 'c++-mode-map)
(makunbound 'c-style-alist)

;;; perhaps do this in a mode hook
;;;(set (make-local-variable 'dabbrev-case-replace) nil)

;;; Note, dabbrev is best upgraded to Lindberg's version 5.7
;;; Emacs 19.28 and earlier need to be updated with the dabbrev.el
;;; file available in the elisp archives at ftp://archive.cis.ohio-state.edu
(setq dabbrev-case-fold-search nil)
(setq dabbrev-case-replace nil)

(if (< emacs-minor-version 29)
    (autoload 'ff-find-other-file "find-file" 
      "Find the header or source file corresponding to this file.
Being on a `#include' line pulls in that file." t))

(autoload 'python-mode "python-mode" "Python editing mode." t)
(autoload 'follow-mode "follow" "Follow mode for side-by-side editing" t)
(autoload 'db-find-file "database" "EDB database package" t)
(autoload 'gene-mode "gene-mode" "Coyote Gene editing mode" t)
(autoload 'sh-mode "sh-script" "Shell-Script editing mode" t)
(autoload 'sh-or-other-mode "sh-script" "Shell-Script editing mode" t)
(autoload 'c++-mode "cc-mode" "C++ Editing Mode" t)
(autoload 'c-mode   "cc-mode" "C Editing Mode" t)
(autoload 'etalk    "etalk"   "ETalk 0.8b" t)
(autoload 'matlab-mode "matlab-mode" "Enter Matlab mode." t)
;;;(autoload 'tex-mode "tex-mode" "Automatic select TeX or LaTeX mode" t)
;;;(autoload 'plain-tex-mode "tex-mode" "Mode for Plain TeX" t)
;;;(autoload 'latex-mode "tex-mode" "Mode for LaTeX" t)
;;;(autoload 'LaTeX-math-mode    "tex-math"      "Math mode for TeX." t)
(autoload 'outline-minor-mode "outline-m" "Minor Outline Mode." t)
(autoload 'smalltalk-mode "/usr/lib/emacs/19.25/lisp/st.el" "" t)
(autoload 'finger "finger" "" t)
(autoload 'html-helper-mode "html-helper-mode" "Yay HTML" t)

;;; My shift-select re-invented s-region.el, instead, uncomment the
;;; "require" line just after it for shift-key selection like in windows.
;;;(autoload 'shift-select-on "shift-select" "Shift-key selection like in Windows" t)
;;;(require 's-region.el)

(autoload 'shell-strip-ctrl-m "shell" nil t)
(autoload 'desktop-load-default "desktop" nil t)
(autoload 'desktop-read "desktop" nil t)
(autoload 'desktop-save "desktop" nil t)

(autoload 'ediff-buffers "ediff" "Visual interface to diff" t)
(autoload 'ediff  "ediff"  "Visual interface to diff" t)
(autoload 'ediff-files "ediff" "Visual interface to diff" t)
(autoload 'epatch  "ediff"  "Visual interface to patch" t)
(autoload 'ediff-patch-file "ediff" "Visual interface to patch" t)
(autoload 'ediff-patch-buffer "ediff" "Visual interface to patch" t)
(autoload 'epatch-buffer "ediff" "Visual interface to patch" t)
(autoload 'vc-ediff "ediff"
  "Interface to diff & version control via vc.el" t) 
(autoload 'rcs-ediff "ediff"
  "Interface to diff & version control via rcs.el" t)

(autoload 'ispell-word "ispell"
  "Check the spelling of word in buffer." t)
(global-set-key "\e$" 'ispell-word)
(autoload 'ispell-region "ispell"
  "Check the spelling of region." t)
(autoload 'ispell-buffer "ispell"
  "Check the spelling of buffer." t)
(autoload 'ispell-complete-word "ispell"
  "Look up current word in dictionary and try to complete it." t)
(autoload 'ispell-change-dictionary "ispell"
  "Change ispell dictionary." t)

(setq tex-dvi-view-command
      (if (eq window-system 'x) "xdvi" "dvi2tty * | cat -s"))
(setq tex-dvi-print-command "dvips -Php")

(setq auto-mode-alist (mapcar 'purecopy
                              '(("\\.tex$" . latex-mode)
				("\\.bib$" . bibtex-mode)
                                ("\\.txi$" . Texinfo-mode)
				("\\.py$" . python-mode)
                                ("\\.el$" . emacs-lisp-mode)
                                ("\\.a$" . cc-mode)
				("\\.C$" . c++-mode)
				("\\.cpp$" . c++-mode)
				("\\.cc$" . c++-mode)
				("\\.c$" . c-mode)
				("\\.gene$" . gene-mode)
				("\\.hh$" . c++-mode)
				("\\.h$" . c++-mode)
				("\\.inl$" . c++-mode)
				("\\.m$" . matlab-mode)
				("\\.html$" . html-helper-mode)
				("\\.stk$" . scheme-mode)
				("\\.stklos$" . scheme-mode)
				("\\.sc$" . scheme-mode)
				("\\.scm$" . scheme-mode)
				("\\.p$" . Pascal-mode)
				("\\.pl$" . perl-mode)
				("\\.pas$" . Pascal-mode)
				("\\.ol$" . outline-mode)
				("\\.txt$" . text-mode)
				("\\.txt~$" . text-mode)
				("/bin/" . sh-or-other-mode)
				("\\.sh$\\|/\\.\\(profile\\|zshrc\\|zshrc-interactive\\|zshrc-compctl\\|zshenv\\|bash_profile\\|login\\|bash_login\\|logout\\|bash_logout\\|bashrc\\|t?cshrc\\|xinitrc\\|startxrc\\|xsession\\)$" . sh-mode)
				("\\.st$" . smalltalk-mode))))

;;; The following function corrects the auto-mode-list var dynamically,
;;; adding a mode function or changing the mode function if it already
;;; exists.  After note from net.emacs, by P. Vijay (vijay@topaz.ARPA)
;;;
(defun Fix-Auto-Mode-Alist (file-name-pattern major-mode-function)
  "Searches auto-mode-alist for a cons pair with FILE-NAME-PATTERN
as its car. If found, MAJOR-MODE-FUNCTION is rplacd in place. If
not found, (FILE-NAME-PATTERN . MAJOR-MODE-FUNCTION) is added
to auto-mode-alist."
  (interactive "sFile name pattern (regexp): \nSMajor mode function: \n")
  (let ((temp (assoc file-name-pattern auto-mode-alist)))
    (if temp
	(setq auto-mode-alist
	      (nconc (list (cons file-name-pattern major-mode-function))
		     auto-mode-alist))
      (setq auto-mode-alist
	    (nconc auto-mode-alist 
		   (list (cons file-name-pattern major-mode-function)))))))

(defun my-info-mode-hook ()
  (setq Info-directory-list
	(append (mapcar 'expand-file-name
			'("~/info"))
		Info-directory-list))
  (setq Info-enable-edit nil))

(add-hook 'Info-mode-hook 'my-info-mode-hook)

(setq tab-stop-list
      '(4 8 12 16 20 24 28 32 36 40 44 48 52 56 60 64 68 72 80 88 96 104 112 120))


(defun Ctl-C-prefix ()
  Ctl-C-keymap)
(setq Ctl-C-keymap (make-keymap))
(global-set-key "\C-c" (Ctl-C-prefix))
 
(define-key Ctl-C-keymap "m" 'LaTeX-math-mode)
(define-key Ctl-C-keymap "o" 'outline-minor-mode)
 
(make-variable-buffer-local 'outline-prefix-char)
(setq-default outline-prefix-char "\C-l")
(make-variable-buffer-local 'outline-regexp)
(setq-default outline-regexp "[*\l]+")
(make-variable-buffer-local 'outline-level-function)
(setq-default outline-level-function 'outline-level-default)


(setq completion-ignored-extensions '(".a" ".aux" ".bbl" ".blg" ".dvi" ".elc"
				      ".lof" ".log" ".lot" ".o" ".toc" "~"
				      ".ky" ".pg" ".pgs" ".toc" ".tp" ".vr"))

(defun title-set-title ()
  "Set title to current's buffer \\[buffer-file-name] name
or to \\[buffer-name if it has no file"
  (let ((name (format "emacs: %s"
		      (buffer-name (current-buffer)))))
    (modify-frame-parameters (selected-frame)
			     (list (cons 'name name)))))

(setq post-command-hook (cons 'title-set-title post-command-hook))


(defun page-down-small ()
    "Move the text on the page down by one line only."
    (interactive)
    (scroll-down 1))

(defun page-up-small ()
    "Move the text on the page up by one line only."
    (interactive)
    (scroll-up 1))

(defalias 'bookmark-jump-f12
  (read-kbd-macro "M-x bookmark-jump RET f12 RET"))
(defalias 'bookmark-jump-f14
  (read-kbd-macro "M-x bookmark-jump RET f14 RET"))
(defalias 'bookmark-jump-f16
  (read-kbd-macro "M-x bookmark-jump RET f16 RET"))
(defalias 'bookmark-jump-f18
  (read-kbd-macro "M-x bookmark-jump RET f18 RET"))
(defalias 'bookmark-jump-f20
  (read-kbd-macro "M-x bookmark-jump RET f20 RET"))

(defun kill-backward-line ()
  (interactive)
  (beginning-of-line)
  (kill-line))

(defun big-font ()
  (interactive)
  (set-default-font "10x20"))

(defun small-font ()
  (interactive)
  (set-default-font "6x13"))

(if (>= emacs-minor-version 29)
    (progn
      (define-key global-map [f12] (function (lambda () "" (interactive)
					       (bookmark-set "f12") 
					       (message "Set Bookmark F12"))))
      (define-key global-map [f14] (function (lambda () "" (interactive)
					       (bookmark-set "f14") 
					       (message "Set Bookmark F14"))))
      (define-key global-map [f16] (function (lambda () "" (interactive)
					       (bookmark-set "f16") 
					       (message "Set Bookmark F16"))))
      (define-key global-map [f18] (function (lambda () "" (interactive)
					       (bookmark-set "f18") 
					       (message "Set Bookmark F18"))))
      (define-key global-map [f20] (function (lambda () "" (interactive)
					       (bookmark-set "f20") 
					       (message "Set Bookmark F20")))))
  (defalias 'bookmark-set-f12 
    (read-kbd-macro "M-x bookmark-set RET f12 RET ESC ESC (message \"Set BM1\") RET"))
  (defalias 'bookmark-set-f14
    (read-kbd-macro "M-x bookmark-set RET f14 RET ESC ESC (message \"Set BM2\") RET"))
  (defalias 'bookmark-set-f16
    (read-kbd-macro "M-x bookmark-set RET f16 RET ESC ESC (message \"Set BM3\") RET"))
  (defalias 'bookmark-set-f18
    (read-kbd-macro "M-x bookmark-set RET f18 RET ESC ESC (message \"Set BM4\") RET"))
  (defalias 'bookmark-set-f20
    (read-kbd-macro "M-x bookmark-set RET f20 RET ESC ESC (message \"Set BM5\") RET"))
  (define-key global-map [f12] 'bookmark-set-f12)
  (define-key global-map [f14] 'bookmark-set-f14)
  (define-key global-map [f16] 'bookmark-set-f16)
  (define-key global-map [f18] 'bookmark-set-f18)
  (define-key global-map [f20] 'bookmark-set-f20))


(define-key global-map [f11] 'bookmark-jump-f12)
(define-key global-map [f13] 'bookmark-jump-f14)
(define-key global-map [f15] 'bookmark-jump-f16)
(define-key global-map [f17] 'bookmark-jump-f18)
(define-key global-map [f19] 'bookmark-jump-f20)

(define-key minibuffer-local-map [down] 'next-history-element)
(define-key minibuffer-local-map [up] 'previous-history-element)

(define-key minibuffer-local-ns-map [down] 'next-history-element)
(define-key minibuffer-local-ns-map [up] 'previous-history-element)

(define-key global-map [C-insert] 'kill-ring-save-deactivate-mark)
(define-key global-map "\M-w" 'kill-ring-save-deactivate-mark)

(define-key global-map [S-insert] 'yank)
(define-key global-map "\M-\C-Q" 'call-last-kbd-macro)
(define-key global-map "\M-n" 'next-file)
(define-key global-map "\M--" 'previous-multiframe-window)
(define-key global-map "\M-`" 'kill-buffer)
(define-key global-map "\M-b" 'switch-to-buffer)
(define-key global-map "\M-g" 'goto-line)
(define-key global-map [C-backspace] 'backward-kill-word)
(define-key global-map [S-M-prior] 'page-down-small)
(define-key global-map [S-M-next] 'page-up-small)
(define-key global-map [home] 'beginning-of-line)
(define-key global-map [end] 'end-of-line)
(define-key global-map [C-tab] 'complete-tag)
(define-key global-map [C-left] 'backward-word)
(define-key global-map [C-right] 'forward-word)
(define-key global-map [C-home] 'beginning-of-buffer)
(define-key global-map [C-end] 'end-of-buffer)
(define-key global-map [del] 'delete-char)

(define-key global-map [M-f2] 'delete-other-windows)
(define-key global-map [f2] 'isearch-forward-regexp)
(define-key global-map [f3] 'replace-regexp)
(define-key global-map [f5] 'search-forward)
(define-key global-map [f6] 'replace-string)
(define-key global-map [M-f5] 'search-backward)
(define-key global-map [M-f6] 'query-replace)
(define-key global-map [f7] 'start-kbd-macro)
(define-key global-map [f8] 'end-kbd-macro)
(define-key global-map [f9] 'tags-search)
(define-key global-map [f10] 'execute-extended-command)
(define-key global-map [f35] 'tags-loop-continue)
(define-key global-map [C-M-insert] 'another-line)
(define-key ctl-x-map "x" 'double-line)
(define-key ctl-x-map "c" 'compile)
(define-key global-map [C-f9] 'recompile)
(define-key global-map [A-down] 'move-line-down)
(define-key global-map [A-up] 'move-line-up)
(define-key global-map [A-up] 'move-line-up)
(define-key global-map [C-up] 'backward-paragraph)
(define-key global-map [C-down] 'forward-paragraph)
(define-key global-map [?\C-)] 'blink-matching-open)
(define-key global-map [C-M-backspace] 'kill-backward-line)
(define-key global-map [A-insert] 'copy-line)
(define-key global-map "\C-\\" 'backward-delete-word-part)
(define-key global-map "\C-x\C-w" 'safe-write-file)
(define-key global-map [C-M-S-return] 'big-font)
(define-key global-map [M-S-return] 'small-font)
(define-key global-map [M-S-up] 'enlarge-window)
(define-key global-map [M-S-down] 'shrink-window)
(define-key global-map "\C-xv\\" 'vc-diff-and-checkin)
(define-key global-map "\C-\M-h" 'switch-to-corresponding-file)
(define-key global-map "\C-c\C-k" 'kill-temporary-buffers)
(define-key global-map [?\M-:] 'eval-expression)
(define-key global-map "\C-cf" 'fax-send-file)

(if (>= emacs-minor-version 29)
    (define-key key-translation-map [f1] nil))

(setq compilation-window-height 15)
(setq interprogram-cut-function nil)
(setq interprogram-paste-function nil)

(defun vc-diff-and-checkin ()
  "Do a vc-diff in one window, checkin log in the other"
  (interactive)
  (vc-diff nil)
  (rename-buffer "*vc-diff*" t)
  (let (diff-buffer-name (buffer-name))
    (vc-next-action nil)
    (shrink-window-if-larger-than-buffer)
    (switch-to-buffer-other-frame diff-buffer-name)))

(defun kill-temporary-buffers()
  "Remove unmodified buffers with *XXXXX* pattern"
  (interactive)
  (if (not (yes-or-no-p "Kill all temporary buffers? "))
      ()
    (save-excursion
      (mapcar '(lambda (b)
		 (if (null (string-match "\\*.*\\*" (buffer-name b)))
		     ()
		   ;;; matches
		   (kill-buffer (buffer-name b))))
	      (buffer-list)))
    (message "Killed all temporary buffers")))
  

(defun kill-ring-save-deactivate-mark (BEGIN END)
  "Same as kill-ring-save, but also deactivate the mark"
  (interactive "r")
  (setq deactivate-mark t)
  (kill-ring-save BEGIN END))

(defun new-extension (file-name extension)
  "Return file-name with the new-extension, extension."
  (string-match "^\\(.*\\.\\)[^\\.\n]*$" file-name)
  (concat (substring file-name 0 (match-end 1)) extension))

(defun switch-to-corresponding-file (extension)
  (interactive "sExtension to find: ")
  (find-file (new-extension (buffer-file-name) extension)))

(defun repname ()
  (interactive)
  (call-process "repname" nil (current-buffer))
  (delete-backward-char 1))

;;; Use M-b to cycle through the buffers!
(define-key global-map "\M-b" 'bury-or-raise-buffer)

(add-hook 'emacs-lisp-mode-hook 
	  (function 
	   (lambda () 
	     (define-key emacs-lisp-mode-map [C-tab] 'lisp-complete-symbol))))

(defun desktop-start-load ()
  "Start the desktop package and load the desktop."
  (interactive)
  (desktop-load-default)
  (desktop-read))

(defun desktop-start-save ()
  "Start the desktop package and save the desktop."
  (interactive)
  (desktop-load-default)
  (desktop-save default-directory))

(add-hook 'diary-display-hook 'fancy-diary-display)

(defun my-insert-tab ()
  "Insert a tab character at the point"
  (interactive)
  (insert-char 9 1))

(setq text-mode-hook '(lambda ()
;;;			( turn-on-auto-fill)
			( define-key text-mode-map [tab] 'my-insert-tab)
			( setq case-fold-search nil)
			( setq fill-column 72)
			( make-local-variable 'fill-prefix)
			( setq default-tab-width 8)))

(define-key text-mode-map [tab] 'my-insert-tab)

(setq mail-mode-hook '(lambda ()
			( turn-on-auto-fill)
			( setq case-fold-search nil)
			( setq fill-column 72)
			( make-local-variable 'fill-prefix)))

;;;(setq blink-paren-hook
;;;      '(lambda ()
;;;	 (if (looking-at (char-to-string last-input-char))
;;;	     (delete-char 1))
;;;	 (blink-matching-open)))

(setq c++-mode-hook '(lambda ()
		       ( define-key c++-mode-map "{"
			 'self-insert-command )
		       ( define-key c++-mode-map "}"
			 'self-insert-command )
		       ( define-key c++-mode-map ";"
			 'self-insert-command )
		       (c-set-offset 'statement-block-intro 0 'add)
		       (c-set-offset 'knr-argdecl-intro 0 'add)
		       (c-set-offset 'block-open 0 'add)
		       (c-set-offset 'statement-block-intro 0 'add)
		       (c-set-offset 'label '- 'add)
		       (c-set-offset 'statement-cont '+ 'add)
		       (setq c-basic-offset 4)
		       (setq c++-continued-member-init-offset nil)
		       (setq c++-electric-colon 0)
		       (setq c++-empty-arglist-indent nil)
		       (setq c++-friend-offset -4)
		       (setq c-tab-always-indents nil)
		       (setq c-argdecl-indent 4)
		       (setq c-backslash-column 78)
		       (setq c-brace-offset 0)
		       (setq c-continued-statement-offset 4)
		       (setq c-label-offset -2)
		       (setq c-auto-newline 0)
		       (setq c-brace-imaginary-offset 0)
		       (setq c-continued-brace-offset 0)
		       (setq c-indent-level 0)
		       (if (string= (user-login-name) "ndb")
			   ()
			 (define-key c++-mode-map "\C-a"
			   'c-beginning-of-statement)
			 (define-key c++-mode-map "\C-\M-h" 
			   'switch-to-corresponding-file)
			 (define-key c++-mode-map "\C-e"
			   'c-end-of-statement))))

(setq matlab-mode-hook 'my-matlab-mode-hook)

(defun my-matlab-mode-hook ()
  (setq fill-column 76)
  (turn-on-auto-fill))

(defun emacs ()
  "Edit the .emacs.el file."
  (interactive)
  (find-file (expand-file-name "~/.emacs.el"))
  (emacs-lisp-mode)
  (if window-system (hilit-repaint-command 'T)))

(load "ctrl-move")
(load "proto")

(global-set-key "\M-\C-p" 'convert-proto-to-memfn)

;;;If backspace sends C-h, need to uncomment below line
;;;(setcdr (assq 'backspace function-key-map) [127])

;;;make delete key do a suck-in delete instead of a backwards delete
(if (assq 'delete function-key-map)
    (setcdr (assq 'delete function-key-map) [delete]))
(global-set-key [delete] 'delete-char)

(add-hook 'kill-emacs-query-functions
	  (function
	   (lambda () (yes-or-no-p "Really quit Emacs? "))))

(defun shell-command-on-buffer (cmd)
  "Run a shell command on the current buffer"
  (interactive
   (list (read-from-minibuffer "Shell command: "
			       nil nil nil 'shell-command-history)))
  (shell-command-on-region (point-min) (point-max) cmd nil))

(add-hook 'html-helper-load-hook
	  (function (lambda ()
		      (define-key html-helper-mode-map [backspace] 'html-helper-maybe-backward-kill-tag)
		      (define-key html-helper-mode-map [delete] 'html-helper-maybe-forward-kill-tag)
		      (setq html-helper-address-string "")
		      (setq html-helper-do-write-file-hooks 't)
		      (setq html-helper-build-new-buffer 't))))


(defun raise-buffer ()
  (interactive)
  (let ((blist (nreverse (buffer-list))))
    (while (string-match "\\` " (buffer-name (car blist)))
      (setq blist (cdr blist)))
    (if blist
	(switch-to-buffer (car blist)))))

(defun bury-or-raise-buffer (&optional arg)
  "Bury the buffer.  If arg, raise the most-recently-buried one."
  (interactive "P")
  (if arg
      (raise-buffer)
    (bury-buffer)))

(defun double-line ()
  "Doubles line point is on."
  (interactive)
  (save-excursion
    (let ((line
           (buffer-substring
            (progn (beginning-of-line) (point))
            (progn (end-of-line) (point)))))
      (insert "\n" line))))

(defun copy-line ()
  "Copy the current line into the kill-ring"
  (interactive)
  (beginning-of-line)
  (kill-line)
  (hilit-yank 1))

(defun add-spaces-after-keywords ()
  (interactive)
  (goto-char (point-min))
  (while (re-search-forward "\\( \\|	\\)if(" nil t)
    (replace-match "\\1if (" nil nil))
  (goto-char (point-min))
  (while (re-search-forward "\\( \\|	\\)while(" nil t)
    (replace-match "\\1while ("))
  (goto-char (point-min))
  (while (re-search-forward "\\( \\|	\\)for(" nil t)
    (replace-match "\\1for ("))
  (goto-char (point-min))
  (while (re-search-forward "\\( \\|	\\)switch(" nil t)
    (replace-match "\\1switch (")))

(defun vc-delete-stale ()
  "Remove all unlocked files (and buffers) for a fresh build"
  (interactive)
  (let (nonempty (dir default-directory))
    (save-excursion
      (cd dir)
      (vc-file-tree-walk
       (function (lambda (f)
		   (if (vc-registered f)
		       (let ((user (vc-locking-user f)))
			 (if (not user)
			     (progn
			       (kill-buffer (find-file-noselect f))
			       (delete-file f)))))))))))

(defun vc-refresh ()
  "Run shell's rcsfresh cmd to update all non-checked out files"
  (interactive)
  (vc-clear-context)
  (save-excursion
    (set-buffer (generate-new-buffer "*vc-refresh output*"))
    (shell-command "rcsfresh" 't)))


(defun update-all-buffers ()
  "Revert all unmodified buffers from disk."
  (interactive)
  (save-excursion
    (mapcar '(lambda (b)
	       (if (buffer-modified-p b)
		   ()
		 (set-buffer b)
		 (if (buffer-file-name)
		     (revert-buffer 'IGNORE-AUTO 'NOCONFIRM)
		   ())))
	    (buffer-list))))

(defun vc-update ()
  "Refresh and then update buffers"
  (interactive)
  (vc-refresh)
  (update-all-buffers))

(defun Makefile ()
  "Load the file Makefile, and switch to makefile mode"
  (interactive)
  (find-file "Makefile")
  (makefile-mode))

(defun Copyright ()
  "Insert TWN copyright notice, and comment it."
  (interactive)
  (goto-char (point-min))
  (insert "$I")
  (insert "d$\n")
  (goto-char (car (cdr (insert-file-contents "/twn/src/copyright.txt"))))
  (comment-region (point-min) (point)))

(defun safe-write-file (FILENAME)
  "Write current buffer into file FILENAME safely.
First checks if FILENAME exists, and if so, prompts
you before overwriting the file."
  (interactive "FSafe write-file: ")
  (if (or (not (file-exists-p FILENAME))
	  (and (file-exists-p FILENAME)
	       (yes-or-no-p (concat "Really overwrite " FILENAME "? "))))
      (write-file FILENAME))
  ())

;;;(defvar case-fold-search-old nil 
;;;  "Used by backward-delete-word-part internally.
;;;Saves the old value of case-fold-search so it won't permanently
;;;change the case-fold-search variable when doing its work.
;;;Note that this still makes case-fold-search buffer-local so its
;;;behaviour will change slightly if it wasn't buffer-local before.")
;;;
;;; Use "let" construct below instead of this variable.

(defun backward-delete-word-part ()	
  "Delete through the preceding Caps or whitespace."
  (interactive)
  (kill-region 
   (point)
   (let (case-fold-search-old case-fold-search)
     (setq case-fold-search nil)
     (if (string-match "\\s-" (char-to-string (preceding-char)))
	 (progn (re-search-backward "\\S-") (forward-char))
       (re-search-backward "[A-Z:_-]\\|\\s-"))
     (setq case-fold-search case-fold-search-old)
     (point))))

(defun fax-send-file ()
  "Use $FAXSERVER hylafax server to send this file as a fax.
If it is a TeX (or LaTeX) file, check for the .dvi, run dvips, and
then fax the resulting ps file.  Otherwise use enscript to create
a ps file from the buffer, and send that."
  (interactive)
  (let ((dvi-file-name (new-extension (buffer-file-name) "dvi"))
	(ps-file-name (new-extension (buffer-file-name) "ps")))
    (if (string-match ".tex$" (buffer-file-name))
	(progn
	  (if (file-exists-p dvi-file-name)
	      (progn
		(message "Running dvips to create faxable ps file...")
 		(if (and (= (call-process "dvips" nil nil nil "-o" ps-file-name dvi-file-name) 0)
			 (file-exists-p ps-file-name))
		    (fax-send-ps-file ps-file-name)
		  (message "Problem creating .ps file!")))
	    (message ".dvi file does not exist.")))
      ;; Not a .tex file
      (setq ps-file-name (concat (buffer-file-name) ".ps"))
      (message "Running enscript to create faxable ps file...")
      (if (and 
	   (= (call-process "nenscript" nil nil nil "-G" "-p" ps-file-name (buffer-file-name)) 0)
	   (file-exists-p ps-file-name))
	  (fax-send-ps-file ps-file-name)
	(message "Error creating ps file!")))))

(defvar fax-to-table
  '(("Greg"))
  "The list of possible fax recipients.")

(defun update-fax-to-table ()
  "Update the fax-to-table variable with .faxdb info."
  (interactive)
  (shell-command (concat "faxdb-to-elisp " 
			 (expand-file-name "~/.faxdb") " > " 
			 (expand-file-name "~/.faxdb-elisp")))
  (load (expand-file-name "~/.faxdb-elisp")))

(update-fax-to-table)

;;;This is now automatically generated from the above
;;;(setq fax-to-table 
;;;      '(("Greg") ("Arman") ("Scott") ("Nate") ("Cam")
;;;	("Vicia") ("Brown") ("Bermuda") ("Welbourne")
;;;	("Dad") ("Mom")))


(defun fax-send-ps-file (ps-file-name &optional default-to default-comments suppress-coverpage)
  "Send the ps-file-name using sendfax.
May specify a default recipient in argument 'to'.
Also may select a comments paragraph.
Argument suppress-coverpage non-nil means don't make a cover page"
  (let ((recipient (completing-read "Fax to: " fax-to-table nil nil default-to))
	(comments (read-string "Comments: " default-comments))
	;; Assume suppress-coverpage was set elsewhere and passed in
	(coverpage-option (if suppress-coverpage '"-n" '"")))
    (shell-command (concat "sendfax -d '" recipient "' -m " 
			   coverpage-option " -c '" comments "' " 
			   ps-file-name))))
;;; This is ugly since a nil parameter still gets passed to the command -- use shell-command
;;;  (if (= 
;;;	 (call-process "~gjb/bin/listargs" nil (get-buffer-create "*Fax Error*") nil
;;;		       "-d" recipient coverpage-option
;;;		       "-c" (concat "\"" comments "\"")
;;;		       ps-file-name)
;;;	 0)
;;;	(message "Fax has been queued.  Use faxstat to check progress.")
;;;      (message "Fax was not queued!  Error!"))))

;;; Original author: ttn@netcom.com, 28-Jan-1996
(defun another-line ()
  "Copies line, preserving cursor column, and increments any numbers found.
This should probably be generalized in the future."
  (interactive)
  (let* ((col (current-column))
         (bol (progn (beginning-of-line) (point)))
         (eol (progn (end-of-line) (point)))
         (line (buffer-substring bol eol)))
    (beginning-of-line)
    (while (re-search-forward "[0-9]+" eol 1)
      (let ((num (string-to-int (buffer-substring
                                  (match-beginning 0) (match-end 0)))))
        (replace-match (int-to-string (1+ num)))))
    (beginning-of-line)
    (insert line "\n")
    (move-to-column col)))


;;{{{ Interactive Lisp

(defun my-inferior-lisp-mode-hook ()
  (setq comint-input-ring-file-name "~/.ilisp_history")
  (comint-read-input-ring t)
  (make-local-variable 'kill-buffer-hook)
  (add-hook 'kill-buffer-hook 'comint-write-input-ring))

(add-hook 'inferior-lisp-mode-hook 'my-inferior-lisp-mode-hook)

(defadvice inferior-lisp (after my-rename act)
  "The buffer is immediately renamed to *lisp*."
  (rename-buffer "*lisp*" t))

;;}}}

;;; FIX:
;;; Ideas:
;;; switch-to-next-unchecked-in-file-buffer
;;; debug setting while editing

;;; Local Variables:
;;; eval: (defun byte-compile-this-file () (write-region (point-min) (point-max) buffer-file-name nil 't) (byte-compile-file buffer-file-name) nil)
;;; local-write-file-hooks: (byte-compile-this-file)
;;; End:
