;;; shift-select.el
;;; By Greg J. Badros -- Jan 1995
;;; $Id: shift-select.el,v 1.5 1995/04/11 01:27:54 gjb Exp $

(provide 'shift-select)

(defvar kill-ring-temp 'nil
  "Temporary storage for the kill-ring")
(defvar kill-ring-yank-pointer-temp 'nil
  "Temporary storage for the kil-ring-yank-pointer")

(make-variable-buffer-local 'ss-shift-set-mark)
(set-default 'ss-shift-set-mark 'nil)

(defun ss-unset-mark-down ()
  "Turn off the mark and move cursor"
  (interactive)
  (if ss-shift-set-mark
    (setq mark-active 'nil))
  (next-line 1)
  (setq this-command 'next-line))

(defun ss-unset-mark-up ()
  "Turn off the mark and move cursor"
  (interactive)
  (if ss-shift-set-mark
      (setq mark-active 'nil))
  (next-line -1)
  (setq this-command 'previous-line))

(defun ss-unset-mark-left ()
  "Turn off the mark and move cursor"
  (interactive)
  (if ss-shift-set-mark
      (setq mark-active 'nil))
  (backward-char 1))

(defun ss-unset-mark-right ()
  "Turn off the mark and move cursor"
  (interactive)
  (if ss-shift-set-mark
      (setq mark-active 'nil))
  (forward-char 1))

(defun ss-set-mark-down ()
  "Set mark and cursor down"
    (interactive)
    (if mark-active
	()
      (set-mark (point))
      (setq ss-shift-set-mark 't))
    (next-line 1)
    (setq this-command 'next-line))

(defun ss-set-mark-up ()
  "Set mark and cursor up"
    (interactive)
    (if mark-active
	()
      (set-mark (point))
      (setq ss-shift-set-mark 't))
    (next-line -1)
    (setq this-command 'next-line))

(defun ss-set-mark-left ()
  "Set mark and cursor down"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (backward-char 1))

(defun ss-set-mark-right ()
  "Set mark and cursor down"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (forward-char 1))

(defun ss-set-mark-forward-word ()
  "Set mark and forward-word"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (forward-word 1))

(defun ss-set-mark-backward-word ()
  "Set mark and backward-word"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (backward-word 1))

(defun ss-set-mark-beginning-of-buffer ()
  "Set mark and goto top"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (beginning-of-buffer))

(defun ss-set-mark-end-of-buffer ()
  "Set mark and goto bottom"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (end-of-buffer))

(defun ss-set-mark-page-down ()
  "Set mark and page-down"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (scroll-down))

(defun ss-set-mark-page-up ()
  "Set mark and page-up"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (scroll-up))

(defun ss-set-mark-beginning-of-line ()
  "Set mark and goto bol"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (beginning-of-line))

(defun ss-set-mark-end-of-line ()
  "Set mark and page-up"
    (interactive)
    (if mark-active
	()
	(set-mark (point)))
    (end-of-line))

(defun ss-kill-region-if-mark-active ()
  "Kill the region only if the mark is currently active"
  (interactive)
  (if mark-active
      (kill-region (point) (mark))))

(defun ss-kill-region-if-mark-active-no-save ()
  "Kill the region only if mark is active, w/o saving"
  (interactive)
  (if mark-active
      (progn
	(setq kill-ring-temp kill-ring)
	(setq kill-ring-yank-pointer-temp kill-ring-yank-pointer)
	(kill-region (point) (mark))
	(setq kill-ring kill-ring-temp)
	(setq kill-ring-yank-pointer kill-ring-yank-pointer-temp))))

(defun ss-set-mark-command ()
  "Set mark command the old-fashioned way.
Make sure that cursoring around doesn't turn it off."
  (interactive)
  (set-mark (point))
  (setq ss-shift-set-mark 'nil))

(defun shift-select-on ()
  "Turn shift-selection pseudo-mode on."
  (interactive)
  (define-key global-map [down] 'ss-unset-mark-down)
  (define-key global-map [up] 'ss-unset-mark-up)
  (define-key global-map [left] 'ss-unset-mark-left)
  (define-key global-map [right] 'ss-unset-mark-right)
  
  (define-key global-map [S-down] 'ss-set-mark-down)
  (define-key global-map [S-up] 'ss-set-mark-up)
  (define-key global-map [S-left] 'ss-set-mark-left)
  (define-key global-map [S-right] 'ss-set-mark-right)

  (define-key global-map [S-C-left] 'ss-set-mark-backward-word)
  (define-key global-map [S-C-right] 'ss-set-mark-forward-word)
  (define-key global-map [S-C-home] 'ss-set-mark-beginning-of-buffer)
  (define-key global-map [S-C-end] 'ss-set-mark-end-of-buffer)
  (define-key global-map [S-prior] 'ss-set-mark-page-down)
  (define-key global-map [S-next] 'ss-set-mark-page-up)
  (define-key global-map [S-home] 'ss-set-mark-beginning-of-line)
  (define-key global-map [S-end] 'ss-set-mark-end-of-line)
  

  (define-key global-map [S-delete] 'ss-kill-region-if-mark-active)
  
  (define-key global-map [?\C-\ ] 'ss-set-mark-command)
  (define-key global-map "\C-@" 'ss-set-mark-command)
  (define-key global-map [S-C-delete] 'ss-kill-region-if-mark-active-no-save)
  (define-key global-map [C-delete] 'ss-kill-region-if-mark-active-no-save)
  (define-key global-map [C-insert] 'kill-ring-save)
  (define-key global-map [S-insert] 'yank))
